"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const anchor_1 = require("@coral-xyz/anchor");
const web3_js_1 = require("@solana/web3.js");
const fs_1 = __importDefault(require("fs"));
const mangoAccount_1 = require("../../src/accounts/mangoAccount");
const client_1 = require("../../src/client");
const constants_1 = require("../../src/constants");
const I80F48_1 = require("../../src/numbers/I80F48");
const utils_1 = require("../../src/utils");
const CLUSTER_URL = process.env.CLUSTER_URL_OVERRIDE || process.env.MB_CLUSTER_URL;
const SOME_KEYPAIR = process.env.PAYER_KEYPAIR_OVERRIDE || process.env.MB_PAYER_KEYPAIR;
const CLUSTER = process.env.CLUSTER_OVERRIDE || 'mainnet-beta';
const GROUP_PK = 'AKeMSYiJekyKfwCc3CUfVNDVAiqk9FfbQVMY3G7RUZUf';
async function main() {
    const options = anchor_1.AnchorProvider.defaultOptions();
    const connection = new web3_js_1.Connection(CLUSTER_URL, options);
    const someKeypair = web3_js_1.Keypair.fromSecretKey(Buffer.from(JSON.parse(fs_1.default.readFileSync(SOME_KEYPAIR, 'utf-8'))));
    const someWallet = new anchor_1.Wallet(someKeypair);
    const someProvider = new anchor_1.AnchorProvider(connection, someWallet, options);
    const client = client_1.MangoClient.connect(someProvider, CLUSTER, constants_1.MANGO_V4_ID[CLUSTER], {
        idsSource: 'api',
    });
    const group = await client.getGroup(new web3_js_1.PublicKey(GROUP_PK));
    const btcPerpPerpMarket = group.perpMarketsMapByMarketIndex.get(0);
    // e.g. change btc-perp leverage to 5/10x
    btcPerpPerpMarket.maintBaseAssetWeight = I80F48_1.I80F48.fromNumber(0.9);
    btcPerpPerpMarket.initBaseAssetWeight = I80F48_1.I80F48.fromNumber(0.8);
    btcPerpPerpMarket.maintBaseLiabWeight = I80F48_1.I80F48.fromNumber(1.1);
    btcPerpPerpMarket.initBaseLiabWeight = I80F48_1.I80F48.fromNumber(1.2);
    const mangoAccountsWithHealth = (await client.getAllMangoAccounts(group, true))
        .map((a) => {
        return { account: a, health: a.getHealth(group, mangoAccount_1.HealthType.maint) };
    })
        .sort((a, b) => a.health.toNumber() - b.health.toNumber());
    for (const obj of mangoAccountsWithHealth) {
        console.log(`${obj.account.publicKey}: ${(0, utils_1.toUiDecimalsForQuote)(obj.health).toFixed(2)} `);
    }
    process.exit();
}
try {
    main();
}
catch (error) {
    console.log(error);
}
