"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const anchor_1 = require("@coral-xyz/anchor");
const spl_token_1 = require("@solana/spl-token");
const web3_js_1 = require("@solana/web3.js");
const dotenv = __importStar(require("dotenv"));
const fs_1 = __importDefault(require("fs"));
const client_1 = require("../src/client");
const constants_1 = require("../src/constants");
const I80F48_1 = require("../src/numbers/I80F48");
const utils_1 = require("../src/utils");
dotenv.config();
const CLUSTER_URL = process.env.CLUSTER_URL_OVERRIDE || process.env.MB_CLUSTER_URL;
const PAYER_KEYPAIR = process.env.PAYER_KEYPAIR_OVERRIDE || process.env.MB_PAYER_KEYPAIR;
const GROUP_NUM = Number(process.env.GROUP_NUM || 0);
const CLUSTER = process.env.CLUSTER_OVERRIDE || 'mainnet-beta';
async function main() {
    const options = anchor_1.AnchorProvider.defaultOptions();
    const connection = new web3_js_1.Connection(CLUSTER_URL, options);
    const admin = web3_js_1.Keypair.fromSecretKey(Buffer.from(JSON.parse(fs_1.default.readFileSync(PAYER_KEYPAIR, 'utf-8'))));
    const adminWallet = new anchor_1.Wallet(admin);
    const adminProvider = new anchor_1.AnchorProvider(connection, adminWallet, options);
    const client = client_1.MangoClient.connect(adminProvider, CLUSTER, constants_1.MANGO_V4_ID[CLUSTER], { idsSource: 'get-program-accounts' });
    const group = await client.getGroupForCreator(admin.publicKey, GROUP_NUM);
    console.log(`Group ${group.publicKey.toBase58()}`);
    const banks = Array.from(group.banksMapByMint.values()).flat();
    const banksMapUsingTokenIndex = new Map(banks.map((bank) => {
        bank.indexedDepositsByMangoAccounts = (0, I80F48_1.ZERO_I80F48)();
        bank.indexedBorrowsByMangoAccounts = (0, I80F48_1.ZERO_I80F48)();
        bank.serum3Total = (0, I80F48_1.ZERO_I80F48)();
        return [bank.tokenIndex, bank];
    }));
    const mangoAccounts = await client.getAllMangoAccounts(group, true);
    mangoAccounts.map((mangoAccount) => {
        mangoAccount.tokensActive().forEach((token) => {
            const bank = banksMapUsingTokenIndex.get(token.tokenIndex);
            if (token.indexedPosition.isPos()) {
                bank.indexedDepositsByMangoAccounts = bank.indexedDepositsByMangoAccounts.add(token.indexedPosition.mul(banksMapUsingTokenIndex.get(token.tokenIndex).depositIndex));
            }
            if (token.indexedPosition.isNeg()) {
                bank.indexedBorrowsByMangoAccounts = bank.indexedBorrowsByMangoAccounts.add(token.indexedPosition
                    .abs()
                    .mul(banksMapUsingTokenIndex.get(token.tokenIndex).borrowIndex));
            }
        });
        mangoAccount.serum3Active().map((s3a) => {
            const baseBank = group.getFirstBankByTokenIndex(s3a.baseTokenIndex);
            const quoteBank = group.getFirstBankByTokenIndex(s3a.quoteTokenIndex);
            const oo = mangoAccount.serum3OosMapByMarketIndex.get(s3a.marketIndex);
            baseBank.serum3Total = baseBank.serum3Total.add(I80F48_1.I80F48.fromU64(oo.baseTokenTotal));
            quoteBank.serum3Total = quoteBank.serum3Total.add(I80F48_1.I80F48.fromU64(oo.quoteTokenTotal));
        });
    });
    for (const bank of await Array.from(banksMapUsingTokenIndex.values()).sort((a, b) => a.tokenIndex - b.tokenIndex)) {
        const account = await (0, spl_token_1.getAccount)(client.program.provider.connection, bank.vault);
        const vault = I80F48_1.I80F48.fromNumber(Number(account.amount));
        const error = vault
            .sub(bank.indexedDeposits
            .mul(bank.depositIndex)
            .sub(bank.indexedBorrows.mul(bank.borrowIndex)))
            .sub(bank.collectedFeesNative)
            .sub(bank.dust)
            .add(I80F48_1.I80F48.fromU64(bank.feesWithdrawn));
        let res = `${bank.name}`;
        res =
            res +
                `\n ${'error'.padEnd(40)} ${(0, utils_1.toUiDecimals)(error, bank.mintDecimals).toLocaleString()}` +
                `\n ${'vault'.padEnd(40)} ${(0, utils_1.toUiDecimals)(vault, bank.mintDecimals).toLocaleString()}` +
                `\n ${'collected fees'.padEnd(40)} ${(0, utils_1.toUiDecimals)(bank.collectedFeesNative, bank.mintDecimals).toLocaleString()}` +
                `\n ${'fees withdrawn'.padEnd(40)} ${(0, utils_1.toUiDecimals)(bank.feesWithdrawn, bank.mintDecimals).toLocaleString()}` +
                `\n ${'deposits'.padEnd(40)} ${(0, utils_1.toUiDecimals)(bank.indexedDeposits.mul(bank.depositIndex), bank.mintDecimals).toLocaleString()}` +
                `\n ${'deposits (sum over all mango accounts)'.padEnd(40)} ${(0, utils_1.toUiDecimals)(bank.indexedDepositsByMangoAccounts, bank.mintDecimals).toLocaleString()}` +
                `\n ${'borrows'.padEnd(40)} ${(0, utils_1.toUiDecimals)(bank.indexedBorrows.mul(bank.borrowIndex), bank.mintDecimals).toLocaleString()}` +
                `\n ${'borrows (sum over all mango accounts)'.padEnd(40)} ${(0, utils_1.toUiDecimals)(bank.indexedBorrowsByMangoAccounts, bank.mintDecimals).toLocaleString()}` +
                `\n ${'deposits - borrows'.padEnd(40)} ${(0, utils_1.toUiDecimals)(bank.indexedDeposits
                    .mul(bank.depositIndex)
                    .sub(bank.indexedBorrows.mul(bank.borrowIndex)), bank.mintDecimals).toLocaleString()}` +
                `\n ${`serum3 total`.padEnd(40)} ${(0, utils_1.toUiDecimals)(bank.serum3Total, bank.mintDecimals).toLocaleString()}`;
        console.log(`${res}`);
    }
    process.exit();
}
try {
    main();
}
catch (error) {
    console.log(error);
}
