"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const anchor_1 = require("@coral-xyz/anchor");
const web3_js_1 = require("@solana/web3.js");
const fs_1 = __importDefault(require("fs"));
const client_1 = require("../src/client");
const constants_1 = require("../src/constants");
const CLUSTER = process.env.CLUSTER_OVERRIDE || 'mainnet-beta';
const CLUSTER_URL = process.env.CLUSTER_URL_OVERRIDE || process.env.MB_CLUSTER_URL;
const USER_KEYPAIR = process.env.USER_KEYPAIR_OVERRIDE || process.env.MB_PAYER_KEYPAIR;
const GROUP_PK = process.env.GROUP_PK || 'AKeMSYiJekyKfwCc3CUfVNDVAiqk9FfbQVMY3G7RUZUf';
const PERP_MARKET_INDEX = Number(process.env.PERP_MARKET_INDEX);
async function forceClosePerpPositions() {
    const options = anchor_1.AnchorProvider.defaultOptions();
    const connection = new web3_js_1.Connection(CLUSTER_URL, options);
    const user = web3_js_1.Keypair.fromSecretKey(Buffer.from(JSON.parse(process.env.KEYPAIR || fs_1.default.readFileSync(USER_KEYPAIR, 'utf-8'))));
    const userWallet = new anchor_1.Wallet(user);
    const userProvider = new anchor_1.AnchorProvider(connection, userWallet, options);
    const client = await client_1.MangoClient.connect(userProvider, CLUSTER, constants_1.MANGO_V4_ID[CLUSTER], {
        idsSource: 'get-program-accounts',
    });
    const group = await client.getGroup(new web3_js_1.PublicKey(GROUP_PK));
    const pm = group.getPerpMarketByMarketIndex(PERP_MARKET_INDEX);
    if (!pm.reduceOnly) {
        throw new Error(`Unexpected reduce only state ${pm.reduceOnly}`);
    }
    if (!pm.forceClose) {
        throw new Error(`Unexpected force close state ${pm.forceClose}`);
    }
    // Get all mango accounts who have a position in the given market
    const mangoAccounts = (await client.getAllMangoAccounts(group)).filter((a) => a.getPerpPosition(PERP_MARKET_INDEX) !== undefined &&
        a.getPerpPositionUi(group, PERP_MARKET_INDEX) !== 0);
    // Sort descending
    mangoAccounts.sort((a, b) => b.getPerpPositionUi(group, PERP_MARKET_INDEX) -
        a.getPerpPositionUi(group, PERP_MARKET_INDEX));
    let a;
    let b;
    let i = 0, j = mangoAccounts.length - 1;
    // i iterates forward to 2nd last account, and b iterates backward till 2nd account
    while (i < mangoAccounts.length - 1 && j > 0) {
        if (i === j) {
            break;
        }
        a = mangoAccounts[i];
        b = mangoAccounts[j];
        // PerpForceClosePosition ix expects a to be long, and b to short
        const sig = await client.perpForceClosePosition(group, PERP_MARKET_INDEX, a, b);
        console.log(`PerpForceClosePosition ${a.publicKey} and ${b.publicKey} , sig https://explorer.solana.com/tx/${sig}?cluster=${CLUSTER == 'devnet' ? 'devnet' : ''}`);
        a = await a.reload(client);
        b = await b.reload(client);
        // Move to previous account once b's position is completely reduced
        if (b.getPerpPositionUi(group, PERP_MARKET_INDEX) === 0) {
            console.log(`Fully reduced position for ${b.publicKey}`);
            j--;
        }
        // Move to next account once a's position is completely reduced
        if (a.getPerpPositionUi(group, PERP_MARKET_INDEX) === 0) {
            console.log(`Fully reduced position for ${a.publicKey}`);
            i++;
        }
    }
}
forceClosePerpPositions();
