"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const anchor_1 = require("@coral-xyz/anchor");
const web3_js_1 = require("@solana/web3.js");
const fs_1 = __importDefault(require("fs"));
const client_1 = require("../src/client");
const constants_1 = require("../src/constants");
const CLUSTER = process.env.CLUSTER_OVERRIDE || 'mainnet-beta';
const CLUSTER_URL = process.env.CLUSTER_URL_OVERRIDE || process.env.MB_CLUSTER_URL;
const USER_KEYPAIR = process.env.USER_KEYPAIR_OVERRIDE || process.env.MB_PAYER_KEYPAIR;
const GROUP_PK = process.env.GROUP_PK || 'AKeMSYiJekyKfwCc3CUfVNDVAiqk9FfbQVMY3G7RUZUf';
const TOKEN_INDEX = Number(process.env.TOKEN_INDEX);
async function forceWithdrawTokens() {
    const options = anchor_1.AnchorProvider.defaultOptions();
    const connection = new web3_js_1.Connection(CLUSTER_URL, options);
    const user = web3_js_1.Keypair.fromSecretKey(Buffer.from(JSON.parse(process.env.KEYPAIR || fs_1.default.readFileSync(USER_KEYPAIR, 'utf-8'))));
    const userWallet = new anchor_1.Wallet(user);
    const userProvider = new anchor_1.AnchorProvider(connection, userWallet, options);
    const client = await client_1.MangoClient.connect(userProvider, CLUSTER, constants_1.MANGO_V4_ID[CLUSTER], {
        idsSource: 'get-program-accounts',
    });
    const group = await client.getGroup(new web3_js_1.PublicKey(GROUP_PK));
    const forceWithdrawBank = group.getFirstBankByTokenIndex(TOKEN_INDEX);
    if (forceWithdrawBank.reduceOnly != 2) {
        throw new Error(`Unexpected reduce only state ${forceWithdrawBank.reduceOnly}`);
    }
    if (!forceWithdrawBank.forceWithdraw) {
        throw new Error(`Unexpected force withdraw state ${forceWithdrawBank.forceWithdraw}`);
    }
    // Get all mango accounts with deposits for given token
    const mangoAccountsWithDeposits = (await client.getAllMangoAccounts(group)).filter((a) => a.getTokenBalanceUi(forceWithdrawBank) > 0);
    for (const mangoAccount of mangoAccountsWithDeposits) {
        const sig = await client.tokenForceWithdraw(group, mangoAccount, TOKEN_INDEX);
        console.log(` tokenForceWithdraw for ${mangoAccount.publicKey}, owner ${mangoAccount.owner}, sig https://explorer.solana.com/tx/${sig}?cluster=${CLUSTER == 'devnet' ? 'devnet' : ''}`);
    }
}
forceWithdrawTokens();
