"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const web3_js_1 = require("@solana/web3.js");
const mangoAccount_1 = require("./mangoAccount");
const bn_js_1 = __importDefault(require("bn.js"));
const utils_1 = require("../utils");
const chai_1 = require("chai");
const I80F48_1 = require("../numbers/I80F48");
describe('Mango Account', () => {
    const mangoAccount = new mangoAccount_1.MangoAccount(web3_js_1.PublicKey.default, web3_js_1.PublicKey.default, web3_js_1.PublicKey.default, [], web3_js_1.PublicKey.default, 0, false, false, new bn_js_1.default(0), new bn_js_1.default(0), new bn_js_1.default(100), new bn_js_1.default(0), new bn_js_1.default(0), new bn_js_1.default(0), new bn_js_1.default(0), 0, [], [], [], [], [], new Map());
    it('test calculateEquivalentSourceAmount', (done) => {
        const mockedSOLBank = {
            mintDecimals: 9,
            price: I80F48_1.I80F48.fromNumber(0.09870201707999726),
        };
        const SOLDepositLimitLeft = new bn_js_1.default((0, utils_1.toNative)(0.5, mockedSOLBank.mintDecimals));
        const mockedUSDCBank = {
            mintDecimals: 6,
            price: I80F48_1.I80F48.fromNumber(1.0000536899999979),
        };
        const USDCDepositLimitLeft = new bn_js_1.default((0, utils_1.toNative)(1, mockedUSDCBank.mintDecimals));
        const maxSourceForUSDCTarget = mangoAccount.calculateEquivalentSourceAmount(mockedSOLBank, mockedUSDCBank, USDCDepositLimitLeft);
        const maxSourceForSOLTarget = mangoAccount.calculateEquivalentSourceAmount(mockedUSDCBank, mockedSOLBank, SOLDepositLimitLeft);
        // Expected u can sell max 0.01 sol for 1 USDC
        (0, chai_1.expect)((0, utils_1.toUiDecimals)(maxSourceForUSDCTarget, mockedSOLBank.mintDecimals).toFixed(2) === '0.01').to.be.true;
        // Expected u can buy max of 0.49 SOL for 49 USDC
        (0, chai_1.expect)((0, utils_1.toUiDecimals)(maxSourceForSOLTarget, mockedUSDCBank.mintDecimals).toFixed(0) === '49').to.be.true;
        done();
    });
});
